#include <stdio.h>
#include <sys/timeb.h>
#include <sys/time.h>
#include <time.h>
#include <string.h>

#include "CameraDevice.h"

// return timestamp in ms from midnight
unsigned long getTimeStamp()
{
#ifdef LINUX
        struct timeval tv;
        gettimeofday(&tv,0);
        return (unsigned long)(tv.tv_sec%86400)*1000+(unsigned long)tv.tv_usec/1000;
#else
        struct timeb timebuffer;
        ftime( &timebuffer );
        unsigned short  millitm = timebuffer.millitm;
        time_t sectm = timebuffer.time;
        return (unsigned long)(sectm%86400)*1000+(unsigned long)millitm;
#endif
}

void RGBImage::savePPM(const char *filename)
{
	FILE *fp;
    long size;

   	if (filename == NULL) return;
   	
    if ((fp = fopen(filename, "wb")) == NULL) {
         fprintf(stderr, "Error writing the file %s in RGBImage::savePPM() \n",
            	filename);
         return;
    }

	// Write the header information to the PGM file.
   	fprintf(fp, "P6\n%d %d\n", width, height);
   	//if(comment != NULL)
    //	if(strlen(comment) <= 70) fprintf(fp, "# %s\n", comment);
   	fprintf(fp, "%d\n", 255);
	
	// Write the image data to the file.
	size = (long)width * height * bpp;
	fwrite(data,size,1,fp);
   
   fclose(fp);
   return;
}

bool RGBImage::loadPPM(const char *filename)
{
   FILE *fp;
   char buf[180];
   int size;

   if (filename == NULL)
     return false;

   if((fp = fopen(filename, "rb")) == NULL){
     fprintf(stderr, "Error reading the file %s in read_ppm_image().\n",
             filename);
     return false;
   }

   fgets(buf, 180, fp);
   if(strncmp(buf,"P6",2) != 0){
      fprintf(stderr, "The file %s is not in PPM format in ", filename);
      fprintf(stderr, "read_ppm_image().\n");
      if(fp != stdin) fclose(fp);
      return false;
   }

   do{ fgets(buf, 70, fp); }while(buf[0] == '#');  /* skip all comment lines */
   sscanf(buf, "%d %d", &width, &height);
   do{ fgets(buf, 70, fp); }while(buf[0] == '#');  /* skip all comment lines */

	if (data)
		free_mem();
	alloc_mem(width,height);

   size = width*height; int r;
   if ((r=fread(data,3,size,fp))<size) {
      fprintf(stderr, "Warning: RGBImage::loadPPM read only %d/%d from file %s.\n",r,size,filename);
   }

   fclose(fp);
   return true;
}


