#include "imagestereograbber.h"
#include <dirent.h>
#include <math.h>

imagestereograbber::imagestereograbber(const string fn, const string ffn, const int kinect_number, int fps, int prev_offset, CvSize dim)
{
    this->d = d;
    this->fps = fps;
    this->prev_offset = prev_offset;
    this->resize = resize;
    this->fn = fn;
    prev_counter = 0;
    time_scale = 1000/this->fps;

    this->sequenceName = fn;

    struct dirent **direct;
    int n = scandir(sequenceName.c_str(), &direct, 0, alphasort);


    if(n == 0) {

        std::cout << "Unable to open directory!" << std::endl;
        exit(-1);

    } else {
        for(int i = 0; i < n; ++i) {
            if(strstr(direct[i]->d_name,".jpg")) {

                filename.push_back(std::string(direct[i]->d_name));

            }

        }
    }

    if(filename.size() == 0) {
        std::cout << "Error: No files found!" << std::endl;
        abort();
    }

    int pos = filename[0].find("_RGB_");

    baseName = filename[0].substr(0, pos);


    img_path = fn + "/" + baseName + "_RGB_";
    depth_path = fn + "/" + baseName + "_DEPTH_";


    pos = filename[0].find_last_of("_");
    string first_frame_string = filename[0].substr(pos+1);
    std::string fr = filename[0].substr(0, pos);
    int pos_1 = fr.find_last_of("_");
    istringstream(fr.substr(pos_1+1, pos)) >> frameNumber;
    img_path = img_path + fr.substr(pos_1+1, pos) + "_";
    depth_path += fr.substr(pos_1+1, pos);


    pos = first_frame_string.find(".");
    first_frame_string = first_frame_string.substr(0, pos);

    IplImage* img = cvLoadImage(std::string(img_path + first_frame_string + ".jpg").c_str());
    if( !img ) {
        cerr<<"unable to read image "<<img_path<<endl;
        cerr<<"aborting..."<<endl;
        abort();
    }

    cv::Mat depthTemp = cv::imread(std::string(depth_path + ".png").c_str(), CV_16UC1);
    if(!depthTemp.data) {
        cerr<<"unable to read image depth  "<< depth_path <<endl;
        cerr<<"aborting..."<<endl;
        abort();
    }

    if(resize) {
        width = dim.width;
        height = dim.height;
    }
    else {
        width = img->width;
        height = img->height;
    }


    lx = 16;
    ly = 30;
    rx = 610;
    ry = depthTemp.rows - 1;


    cvReleaseImage(&img);

    CvSize frameSize = cvSize(width, height);

    next_img_path.assign(std::string(img_path + first_frame_string + ".jpg"));
    next_depth_path.assign(std::string(depth_path + ".png"));

    frame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);
    prevFrame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);
    disparityImage = cv::Mat(cv::Size(width, height), CV_16UC1);

    disparity = new short[height*width*sizeof(short)];

    timestamp = 0;

    translation = theta = h = 0;

    std::string line;
    ifstream infile(ffn.c_str());
    while (std::getline(infile, line))
    {
        std::istringstream iss(line);
        std::string temp;
        double tt;
        if(!(iss >> temp >> tt)) break;
        if(strstr(temp.c_str(), "height") != NULL) {
            h = tt;
            std::cout << "height: " << h << std::endl;
        } else if(strstr(temp.c_str(), "angle") != NULL) {
            theta = tt;
            std::cout << "theta: " << theta << std::endl;
        } else if (strstr(temp.c_str(), "translation") != NULL) {
            translation = tt;
            std::cout << "translation: " << translation << std::endl;
        }
    }

    if(translation == 0 && theta == 0 && h == 0) {
        std::cout << "Error: Load Kinect Parameters" << std::endl;
        exit(-1);
    }
    infile.close();

    std::stringstream ss;
    ss << "homography" << kinect_number << ".txt";
    homographyFilename = ss.str();

    homography = cv::Mat::zeros(cv::Size(3,3), CV_64F);

    int i = 0;

    std::cout << "h: " << homographyFilename.c_str() << std::endl;
    ifstream hfile(homographyFilename.c_str());
    while (std::getline(hfile, line))
    {
        std::istringstream iss(line);
        double a, b, c;
        if(!(iss >> a >> b >> c)) break;
        homography.at<double>(i,0) = a;
        homography.at<double>(i,1) = b;
        homography.at<double>(i,2) = c;
        ++i;
    }


    std::cout << homography << std::endl;

    ox = oz = 0;
    oy = h;

}

imagestereograbber::~imagestereograbber() {
    cvReleaseImage(&frame);
    cvReleaseImage(&prevFrame);
    delete[] disparity;
    disparity = NULL;
}



cv::Mat imagestereograbber::getDepth16bit() {
    return depth16bitNew;
}

void imagestereograbber::get3DPoint(int x, int y, double *wx, double *wy, double *wz) {
    double ref_pix_size = 0.104200;
    double ref_distance = 120.000000;
    double ly;
    double c = cos(CV_PI*theta/180);
    double s = sin(CV_PI*theta/180);
    double z = depth16bitNew.at<ushort>(y, x);
    double factor = 2 * ref_pix_size * z / ref_distance;
    *wx = (double)(x - width/2) * factor;
    double Y = ((double)(y - height/2) * factor)*-1;

    double temp = Y;
    Y = z;
    z = temp;
    z -= oz;
    ly = Y - oy;

    //ROTAZIONE
    Y = ly * c - z * s;
    z = ly * s + z * c;

    *wy = Y + oy;
    *wz = z + oz + translation;
}


bool imagestereograbber::getData() {
    prev_counter++;
    if(prev_counter == prev_offset) {
        cvCopy(frame, prevFrame);
        prev_counter = 0;
    }

    IplImage* loadedImage = cvLoadImage(next_img_path.c_str());
    if(loadedImage == NULL) {
    	cout << "\nUnable to load " << next_img_path << endl;

#if __linux__
    	int pos = img_path.find_last_of("/");
#else
    	int pos = img_path.find_last_of("\");
#endif

    	string name = img_path.substr(pos+1);

    	//find next image if any
    	vector<string>::iterator it;
    	it = filename.begin();
    	int index = -1;
    	int counter = 0;
    	for(; it != filename.end(); ++it) {
    		if(*it == name) {
    			index = counter;
    			break;
    		}
    		counter++;
    	}

    	if(index == -1 || index == (filename.size()-1)) {
    		return false;
    	}
    	else {
    		string s = img_path.substr(0, pos+1);
    		next_img_path = s + filename[index+1];
    		loadedImage = cvLoadImage(next_img_path.c_str());
    		if(loadedImage == NULL) {
    		    cout << "\nUnable to load " << next_img_path << endl;
    		    return false;
    		}
    		pos = filename[index+1].find_last_of("_");
    		int pos2 = filename[index+1].find_last_of(".");
    		string number = filename[index+1].substr(pos+1,pos2-pos-1);
    		next_depth_path = depth_path + number + ".png";
    		istringstream(number) >> frameNumber;
    	}
    }

    disparityImage = cv::imread(next_depth_path.c_str(), CV_LOAD_IMAGE_UNCHANGED);
    if(disparityImage.data == NULL) {
    	cout << "\nUnable to load " << next_depth_path << endl;
        return false;
    }

    depth16bitNew = disparityImage.clone();

    depth8bit = cv::Mat::zeros(disparityImage.size(), CV_8UC1);

    disparityImage.convertTo(depth8bit, CV_8UC1, 255.0/2048.0);

    int offset = 0;
	for(int j = 0; j < height; ++j) {
		for(int i = 0; i < width; ++i) {
				memcpy(&disparityImage.data[offset],&disparity[j * width + i], sizeof(short));
				offset += sizeof(short);
		}
	}

    cvCopy(loadedImage, frame);
    cvReleaseImage(&loadedImage);

    img_path.assign(next_img_path);

    prevTimestamp = timestamp;

    frameNumber++;

    std::stringstream searchTime;
    searchTime << "_" << frameNumber << "_";

    int idx = -1;
    for(int i = 0; i < filename.size(); ++i) {
        if(strstr(filename[i].c_str(), searchTime.str().c_str()) != NULL) {
            idx = i;
            break;
        }
    }

    int pos_1 = filename[idx].find(searchTime.str().c_str()) + searchTime.str().length();
    int pos_2 = filename[idx].find(".jpg") - pos_1;

    timestamp = atoi(filename[idx].substr(pos_1, pos_2).c_str());

    std::stringstream out_next_path, out_next_path_depth;
    out_next_path << fn << "/" << baseName << "_RGB_" << frameNumber <<  "_" << timestamp << ".jpg" ;
    out_next_path_depth << fn << "/" << baseName << "_DEPTH_" << frameNumber << ".png";

    next_img_path = out_next_path.str();
    next_depth_path = out_next_path_depth.str();

    return true;
}

unsigned long imagestereograbber::getTimestamp() {
    return timestamp;
}

unsigned long imagestereograbber::getPrevTimestamp() {
    return prevTimestamp;
}

void imagestereograbber::reset() {

    img_path = fn + "/" + baseName + "_RGB_";
    depth_path = fn + "/" + baseName + "_DEPTH_";


    int pos = filename[0].find_last_of("_");
    string first_frame_string = filename[0].substr(pos+1);
    std::string fr = filename[0].substr(0, pos);
    int pos_1 = fr.find_last_of("_");
    istringstream(fr.substr(pos_1+1, pos)) >> frameNumber;
    img_path = img_path + fr.substr(pos_1+1, pos) + "_";
    depth_path += fr.substr(pos_1+1, pos);


    pos = first_frame_string.find(".");
    first_frame_string = first_frame_string.substr(0, pos);

	cout << "RESET first_frame_string " << first_frame_string << endl;

    next_img_path.assign(std::string(img_path + first_frame_string + ".jpg"));
    next_depth_path.assign(std::string(depth_path /*+ first_frame_string*/ + ".png"));

//    std::cout << next_img_path << std::endl;
//    std::cout << next_depth_path << std::endl;

	timestamp = 0;
}


