#include "kinectgrabber.h"

kinectgrabber::kinectgrabber(const int width, const int height, const int kinect_number, const string ffn, int fps, int prev_offset) {
    this->width = width;
    this->height = height;
    this->prev_offset = prev_offset;
    this->fps = fps;
    time_scale = 1000/this->fps;

    //DEPTH DIMENSIONS
    lx = 16;
    ly = 30;
    rx = 610;
    ry = height - 1;

    translation = theta = h = 0;

    //KINECT PARAMETERS
    std::string line;
    ifstream infile(ffn.c_str());
    while (std::getline(infile, line))
    {
        std::istringstream iss(line);
        std::string temp;
        double tt;
        if(!(iss >> temp >> tt)) break;
        if(strstr(temp.c_str(), "height") != NULL) {
            h = tt;
            std::cout << "height: " << h << std::endl;
        } else if(strstr(temp.c_str(), "angle") != NULL) {
            theta = tt;
            std::cout << "theta: " << theta << std::endl;
        } else if (strstr(temp.c_str(), "translation") != NULL) {
            translation = tt;
            std::cout << "translation: " << translation << std::endl;
        }
    }

    if(translation == 0 && theta == 0 && h == 0) {
        std::cout << "Error: Load Kinect Parameters" << std::endl;
        exit(-1);
    }
    infile.close();

    std::stringstream ss;
    ss << "homography" << kinect_number << ".txt";
    homographyFilename = ss.str();

    homography = cv::Mat::zeros(cv::Size(3,3), CV_64F);

    int i = 0;

    std::cout << "h: " << homographyFilename.c_str() << std::endl;
    ifstream hfile(homographyFilename.c_str());
    while (std::getline(hfile, line))
    {
        std::istringstream iss(line);
        double a, b, c;
        if(!(iss >> a >> b >> c)) break;
        homography.at<double>(i,0) = a;
        homography.at<double>(i,1) = b;
        homography.at<double>(i,2) = c;
        ++i;
    }


    std::cout << homography << std::endl;

    ox = oz = 0;
    oy = h;

    depth16bitNew = cv::Mat(cv::Size(width,height),CV_16UC1);
    depth8bit = cv::Mat(cv::Size(width,height),CV_8UC3);
    frameNew = cv::Mat(cv::Size(width,height),CV_8UC3,cv::Scalar(0));

    device = &freenect.createDevice<MyFreenectDevice>(kinect_number);

    device->startVideo();
    device->startDepth();


    if(device->getVideo(frameNew)) {
        //device->getDepth(depth16bitNew);
        //frame = cvCreateImage(cvSize(width,height),8, 3);
        IplImage* img = new IplImage(frameNew);
        if( !img ) {
            cerr<<"unable to read images " <<endl;
            cerr<<"aborting..."<<endl;
            abort();
        }
    }

    frame = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);
    prevFrame = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);
    disparityImage = cv::Mat(cv::Size(width, height), CV_16UC1);

    disparity = new short[height*width*sizeof(short)];

    prev_counter = 0;
    timestamp = getTimestamp();

    frameNumber = 0;
}

kinectgrabber::~kinectgrabber() {
    cvReleaseImage(&frame);
    cvReleaseImage(&prevFrame);
    delete[] disparity;
    disparity = NULL;
}

cv::Mat kinectgrabber::getDepth16bit() {
    return depth16bitNew;
}

bool kinectgrabber::getData() {
    prev_counter++;
    if(prev_counter == prev_offset) {
        cvCopy(frame, prevFrame);
        prev_counter = 0;
    }
    if(device->getVideo(frameNew)) {
                
        //COLOR FRAME
        IplImage* tmp = new IplImage(frameNew);
        cvCopy(tmp, frame);
        cvShowImage("Kinect View - Color Image", frame);
        
        
        //DISPARITY
        device->getDepth(disparityImage);                
        cv::Mat coloredDepth = calcColorDepth();        	         
        cv::imshow("Kinect View - Depth Image", coloredDepth);
        depth16bitNew = disparityImage.clone();
        depth8bit = cv::Mat::zeros(disparityImage.size(), CV_8UC1);
        disparityImage.convertTo(depth8bit, CV_8UC1, 255.0/2048.0);

        int offset = 0;
        for(int j = 0; j < height; ++j) {
            for(int i = 0; i < width; ++i) {
                    memcpy(&disparityImage.data[offset],&disparity[j * width + i], sizeof(short));
                    offset += sizeof(short);
            }
        }
        
        prevTimestamp = timestamp;
        timestamp = getTimestamp();

        frameNumber++;
        
        cv::waitKey(30);

        return true;
    }
    else
        return false;

}

unsigned long kinectgrabber::getTimestamp() {
#ifdef __linux__
        struct timeval tv;
        gettimeofday(&tv,0);
        return (unsigned long)(tv.tv_sec%86400)*1000+(unsigned long)tv.tv_usec/1000;
#else
        struct timeb timebuffer;
        ftime( &timebuffer );
        unsigned short  millitm = timebuffer.millitm;
        time_t sectm = timebuffer.time;
        return (unsigned long)(sectm%86400)*1000+(unsigned long)millitm;
#endif
}

unsigned long kinectgrabber::getPrevTimestamp() {
    return prevTimestamp;
}

void kinectgrabber::reset() {
	
}

cv::Mat kinectgrabber::calcColorDepth() {
	
	cv::Mat depthMat = disparityImage;        
	cv::Mat newDepthMat = cv::Mat::zeros(depthMat.size(), CV_16UC1);

	cv::Mat coloredDepth = cv::Mat::zeros(depthMat.size(), CV_8UC3);
	for(int i = 0; i < depthMat.rows; ++i) {
		for(int j = 0; j < depthMat.cols; ++j) {
			ushort v = depthMat.at<ushort>(i,j);
        	if(v > 4000) {
            	coloredDepth.at<cv::Vec3b>(i,j)[2] = 0;
            	coloredDepth.at<cv::Vec3b>(i,j)[1] = 0;
            	coloredDepth.at<cv::Vec3b>(i,j)[0] = 0;
        	}
        	else if(v > 3500) {
            	coloredDepth.at<cv::Vec3b>(i,j)[2] = 0;
            	coloredDepth.at<cv::Vec3b>(i,j)[1] = 0;
            	coloredDepth.at<cv::Vec3b>(i,j)[0] = 0;
			}
        	else if(v > 3000) {
				coloredDepth.at<cv::Vec3b>(i,j)[2] = 255;
				coloredDepth.at<cv::Vec3b>(i,j)[1] = 255;
				coloredDepth.at<cv::Vec3b>(i,j)[0] = 255;
            	newDepthMat.at<ushort>(i,j) = v;
			}
			else if(v > 2500) {
				coloredDepth.at<cv::Vec3b>(i,j)[2] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[1] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[0] = 255;
            	newDepthMat.at<ushort>(i,j) = v;
			}
			else if(v > 2000) {
				coloredDepth.at<cv::Vec3b>(i,j)[2] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[1] = 255;
				coloredDepth.at<cv::Vec3b>(i,j)[0] = 0;
            	newDepthMat.at<ushort>(i,j) = v;
			}
			else if(v > 1500) {
				coloredDepth.at<cv::Vec3b>(i,j)[2] = 255;
				coloredDepth.at<cv::Vec3b>(i,j)[1] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[0] = 0;
            	newDepthMat.at<ushort>(i,j) = v;
			}
			else if(v > 1000) {
				coloredDepth.at<cv::Vec3b>(i,j)[2] = 255;
				coloredDepth.at<cv::Vec3b>(i,j)[1] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[0] = 255;
            	newDepthMat.at<ushort>(i,j) = v;
			}
			else if(v > 500) {
				coloredDepth.at<cv::Vec3b>(i,j)[2] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[1] = 255;
				coloredDepth.at<cv::Vec3b>(i,j)[0] = 255;
            	newDepthMat.at<ushort>(i,j) = v;
			}
        	else {
				coloredDepth.at<cv::Vec3b>(i,j)[2] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[1] = 0;
				coloredDepth.at<cv::Vec3b>(i,j)[0] = 0;
        	}
		}
	}
	return coloredDepth;
}

