#ifndef MYFREENECTDEVICE_H
#define MYFREENECTDEVICE_H

#include <vector>
#include <cmath>
#include <pthread.h>
#include "libfreenect.hpp"
#include <opencv/cv.h>
#include <opencv/cvaux.h>
#include <opencv/cxcore.h>
#include <opencv/highgui.h>
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"

using namespace std;

#define         FREENECT_FRAME_W   640
#define         FREENECT_FRAME_H   480
#define         FREENECT_FRAME_PIX   (FREENECT_FRAME_H*FREENECT_FRAME_W) // width*height pixels in the image
#define         FREENECT_VIDEO_RGB_SIZE   (FREENECT_FRAME_PIX*3) //3 bytes per pixel

class Mutex_v {
    public:
        Mutex_v() {
            pthread_mutex_init( &m_mutex, NULL );
        }
        void lock() {
            pthread_mutex_lock( &m_mutex );
        }
        void unlock() {
            pthread_mutex_unlock( &m_mutex );
        }

        class ScopedLock {
            Mutex_v & _mutex;
        public:
            ScopedLock(Mutex_v & mutex) : _mutex(mutex) {
                _mutex.lock();
            }
            ~ScopedLock() {
                _mutex.unlock();
            }
        };
    private:
        pthread_mutex_t m_mutex;
};

class MyFreenectDevice : public Freenect::FreenectDevice
{
public:
    MyFreenectDevice(freenect_context *_ctx, int _index);
    void VideoCallback(void* _rgb, uint32_t timestamp);
    void DepthCallback(void* _depth, uint32_t timestamp);
    bool getVideo(cv::Mat& output);
    bool getDepth(cv::Mat& output);
private:
  std::vector<uint8_t> m_buffer_depth;
  std::vector<uint8_t> m_buffer_rgb;
  std::vector<uint16_t> m_gamma;
  cv::Mat depthMat;
  cv::Mat rgbMat;
  cv::Mat ownMat;
  Mutex_v m_rgb_mutex;
  Mutex_v m_depth_mutex;
  bool m_new_rgb_frame;
  bool m_new_depth_frame;

};

#endif // MYFREENECTDEVICE_H
