/*
 *  stereofilegrabber.cpp
 *
 *  Author: Domenico Bloisi
 *  domenicobloisi@yahoo.com
 *  Copyright 2012
 */

#include "stereofilegrabber.h"

StereoFileGrabber::StereoFileGrabber(const char *fn, const char *fini, int prev_offset)
{
	prev_counter = 0;
	this->prev_offset = prev_offset;
	fdev.init(fn, fini);
	sdev = &fdev;

	height = sdev->height;
	width = sdev->width;
	fps = 10;
	CvSize frameSize = cvSize(width, height);

	time_scale = 1000 / fps;

	frameNumber = 0;

	frame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);
	prevFrame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);

	disparity = new short[height*width];

	timestamp = 0;
}

StereoFileGrabber::~StereoFileGrabber()
{
	cvReleaseImage(&frame);
	cvReleaseImage(&prevFrame);

	delete[] disparity;
	disparity = NULL;
}

bool StereoFileGrabber::getData()
{
	prev_counter++;
	if(prev_counter == prev_offset)
	{
		cvCopy(frame, prevFrame);
		prev_counter = 0;
	}

	sdev->getFrame();

	IplImage iplImage = sdev->stereoimage.leftColorImage;

	cvCopy(&iplImage, frame);

	if(!sdev->calcDisparity())
		return false;

	memcpy(disparity, sdev->disparity, width*height*sizeof(short));

	disparityImage = sdev->processDisparity();

	svsstereoimage = sdev->svsstereoimage;

	prevTimestamp = timestamp;
	timestamp = sdev->timestamp;

	frameNumber++;
	return true;
}

unsigned long StereoFileGrabber::getTimestamp() {
	return timestamp;
}

unsigned long StereoFileGrabber::getPrevTimestamp() {
	return prevTimestamp;
}
