/*
 *  stereograbber.cpp
 *
 *  Author: Domenico Bloisi
 *  domenicobloisi@yahoo.com
 *  Copyright 2012
 */

#include "stereograbber.h"

StereoGrabber::StereoGrabber(int width, int height, int fps, bool enableRectification, int prev_offset) {
	prev_counter = 0;
	this->prev_offset = prev_offset;

	capture = NULL;

	this->height = height;
	this->width = width;
	this->fps = fps;
	CvSize frameSize = cvSize(width, height);

	frameNumber = 0;

	frame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);
	prevFrame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);

	disparity = new short[height*width];
	if(disparity == NULL) {
		cout << "unable to allocate disparityImage" << endl;
		exit(EXIT_FAILURE);
	}

	timestamp = getTimestamp();

	vdev.init(width, height, fps, enableRectification);
	sdev = &vdev;
}

StereoGrabber::~StereoGrabber()
{
	cvReleaseImage(&frame);
	cvReleaseImage(&prevFrame);
	cvReleaseCapture(&capture);

	delete[] disparity;
	disparity = NULL;

}

bool StereoGrabber::getData()
{
	prev_counter++;
	if(prev_counter == prev_offset)
	{
		cvCopy(frame, prevFrame);
		prev_counter = 0;
	}

	sdev->getFrame();

	IplImage iplImage = sdev->stereoimage.leftColorImage;

	cvCopy(&iplImage, frame);

	if(!sdev->calcDisparity())
		return false;

	memcpy(disparity, sdev->disparity, width*height*sizeof(short));

	disparityImage = sdev->processDisparity();

	svsstereoimage = sdev->svsstereoimage;

	prevTimestamp = timestamp;
	timestamp = sdev->timestamp;

	frameNumber++;
	return true;
}

unsigned long StereoGrabber::getTimestamp() {
#ifdef __linux__
        struct timeval tv;
        gettimeofday(&tv,0);
        return (unsigned long)(tv.tv_sec%86400)*1000+(unsigned long)tv.tv_usec/1000;
#else
        struct timeb timebuffer;
        ftime( &timebuffer );
        unsigned short  millitm = timebuffer.millitm;
        time_t sectm = timebuffer.time;
        return (unsigned long)(sectm%86400)*1000+(unsigned long)millitm;
#endif
}

unsigned long StereoGrabber::getPrevTimestamp() {
	return prevTimestamp;
}
